@@ -1,4 +1,3 @@
-# Lint as: python3
 """Code and data structures for managing source directives."""
 
 import bisect
@@ -32,7 +31,7 @@ class SkipFileError(Exception):
   """Exception thrown if we encounter "pytype: skip-file" in the source code."""
 
 
-class _LineSet(object):
+class _LineSet:
   """A set of line numbers.
 
   The data structure is optimized to represent the union of a sparse set
@@ -149,9 +148,9 @@ class _VariableAnnotation:
   """Processes a single logical line, looking for a variable annotation."""
 
   @classmethod
-  def start(cls, token):
+  def start(cls, lineno, token):
     self = cls()
-    self.add_token(token)
+    self.add_token(lineno, token)
     return self
 
   def __init__(self):
@@ -160,6 +159,9 @@ class _VariableAnnotation:
     # Set to True when the full annotation has been found, or if we determine
     # that the line does not contain an annotation.
     self.closed = False
+    # Set to the line on which the colon is found. We do not use the line at
+    # which start() is called because the latter may be a blank line.
+    self.start_lineno = None
 
   def _accept(self, token):
     if self.closed:
@@ -168,7 +170,7 @@ class _VariableAnnotation:
     # of the annotation.
     return token.exact_type != tokenize.COMMENT and token.string.strip()
 
-  def add_token(self, token):
+  def add_token(self, lineno, token):
     """Process a token."""
     if not self._accept(token):
       return
@@ -180,7 +182,9 @@ class _VariableAnnotation:
       if token.exact_type != tokenize.NAME or keyword.iskeyword(token.string):
         self.closed = True
     elif len(self._tokens) == 1:
-      if token.exact_type != tokenize.COLON:
+      if token.exact_type == tokenize.COLON:
+        self.start_lineno = lineno
+      else:
         self.closed = True
     elif token.exact_type == tokenize.EQUAL:
       self.closed = True
@@ -204,10 +208,10 @@ def _collect_bytecode(ordered_code):
   return bytecode_blocks
 
 
-class Director(object):
+class Director:
   """Holds all of the directive information for a source file."""
 
-  def __init__(self, src, errorlog, filename, disable):
+  def __init__(self, src, errorlog, filename, disable, python_version):
     """Create a Director for a source file.
 
     Args:
@@ -216,6 +220,7 @@ class Director(object):
           errorlog.
       filename: The name of the source file.
       disable: List of error messages to always ignore.
+      python_version: The target python version.
     """
     self._filename = filename
     self._errorlog = errorlog
@@ -237,7 +242,7 @@ class Director(object):
     for error_name in disable:
       self._disables[error_name].start_range(0, True)
     # Parse the source code for directives.
-    self._parse_source(src)
+    self._parse_source(src, python_version)
 
   @property
   def type_comments(self):
@@ -261,7 +266,7 @@ class Director(object):
   def decorators(self):
     return self._decorators
 
-  def _parse_source(self, src):
+  def _parse_source(self, src, python_version):
     """Parse a source file, extracting directives from comments."""
     f = moves.StringIO(src)
     defs_start = None
@@ -306,16 +311,23 @@ class Director(object):
       if last_function_definition and last_function_definition.contains(lineno):
         pass  # ignore function annotations
       elif not open_variable_annotation:
-        open_variable_annotation = _VariableAnnotation.start(token)
+        open_variable_annotation = _VariableAnnotation.start(lineno, token)
       elif tok in (tokenize.NEWLINE, tokenize.SEMI):
         # NEWLINE indicates the end of a *logical* line of Python code, allowing
         # us to handle annotations split over multiple lines.
         annotation = open_variable_annotation.annotation
         if annotation and open_variable_annotation.closed:
-          self._variable_annotations[lineno] = annotation
+          # In 3.8+, the STORE_* opcode for a multiline variable assignment is
+          # at the first line in the assignment; before that, it is at the last.
+          if python_version >= (3, 8):
+            assert open_variable_annotation.start_lineno
+            annotation_lineno = open_variable_annotation.start_lineno
+          else:
+            annotation_lineno = lineno
+          self._variable_annotations[annotation_lineno] = annotation
         open_variable_annotation = None
       else:
-        open_variable_annotation.add_token(token)
+        open_variable_annotation.add_token(lineno, token)
 
       # Record docstrings.
       if _DOCSTRING_RE.match(line):
@@ -336,8 +348,9 @@ class Director(object):
     is_nested = bool(matches) and matches[0].start(0) > 0
     for m in matches:
       code = line[:col].strip()
-      tool, data = m.groups()
       open_ended = not code
+      tool, data = m.groups()
+      assert data is not None
       data = data.strip()
       if tool == "type":
         self._process_type(lineno, code, data, is_nested, type_comment_set)
@@ -382,8 +395,8 @@ class Director(object):
       try:
         command, values = option.split("=", 1)
         values = values.split(",")
-      except ValueError:
-        raise _DirectiveError("Invalid directive syntax.")
+      except ValueError as e:
+        raise _DirectiveError("Invalid directive syntax.") from e
       # Additional commands may be added in the future.  For now, only
       # "disable" and "enable" are supported.
       if command == "disable":
