@@ -12,7 +12,7 @@ class SysOverlay(overlay.Overlay):
         "version_info": build_version_info
     }
     ast = vm.loader.import_name("sys")
-    super(SysOverlay, self).__init__(vm, "sys", member_map, ast)
+    super().__init__(vm, "sys", member_map, ast)
 
 
 class VersionInfo(abstract.Tuple):
@@ -27,15 +27,14 @@ class VersionInfo(abstract.Tuple):
     return self.pyval[index]
 
 
-def build_version_info(name, vm):
+def build_version_info(vm):
   """Build sys.version_info."""
-  del name
   version = []
   # major, minor
   for i in vm.python_version:
     version.append(vm.convert.constant_to_var(i))
   # micro, releaselevel, serial
   for t in (int, str, int):
-    version.append(
-        vm.convert.primitive_class_instances[t].to_variable(vm.root_cfg_node))
+    version.append(vm.convert.primitive_class_instances[t].to_variable(
+        vm.root_node))
   return VersionInfo(tuple(version), vm)
