@@ -1,4 +1,3 @@
-# Lint as: python2, python3
 """A library for accessing pytype's inferred local types."""
 
 import itertools
@@ -9,15 +8,16 @@ from pytype import analyze
 from pytype import config
 from pytype import errors
 from pytype import load_pytd
+from pytype.ast import visitor
 from pytype.pytd import pytd
 from pytype.pytd import pytd_utils
 from pytype.pytd import visitors
 
 from pytype.tools.traces import source
-from pytype.tools.traces import visitor
 
 _ATTR_OPS = frozenset((
     "LOAD_ATTR",
+    "LOAD_METHOD",
     "STORE_ATTR",
 ))
 
@@ -100,7 +100,7 @@ def _to_pytd(datum, loader, ast):
   return loader.resolve_type(t, ast)
 
 
-class _SymbolMatcher(object):
+class _SymbolMatcher:
   """Symbol matcher for MatchAstVisitor._get_traces.
 
   Allows matching against:
@@ -130,7 +130,7 @@ class _SymbolMatcher(object):
   def match(self, symbol):
     for match in self._matches:
       if isinstance(match, self._PATTERN_TYPE):
-        if match.match(symbol):
+        if match.match(str(symbol)):
           return True
       elif match == symbol:
         return True
@@ -145,7 +145,7 @@ class MatchAstVisitor(visitor.BaseVisitor):
   """
 
   def __init__(self, src_code, *args, **kwargs):
-    super(MatchAstVisitor, self).__init__(*args, **kwargs)
+    super().__init__(*args, **kwargs)
     self.source = src_code
     # In Python versions before 3.7, there is a mismatch between where the ast
     # and bytecode representations think some nodes are located, so we manually
@@ -183,8 +183,8 @@ class MatchAstVisitor(visitor.BaseVisitor):
     method = "match_" + node.__class__.__name__
     try:
       match = getattr(self, method)
-    except AttributeError:
-      raise NotImplementedError(method)
+    except AttributeError as e:
+      raise NotImplementedError(method) from e
     return match(node)
 
   def match_Attribute(self, node):
@@ -215,14 +215,21 @@ class MatchAstVisitor(visitor.BaseVisitor):
             for tr in self._get_traces(
                 node.lineno, _CALL_OPS, name, maxmatch=1, num_lines=5)]
 
-  def match_Ellipsis(self, node):
-    return self._match_constant(node, Ellipsis)
-
   def match_Constant(self, node):
     # As of Python 3.8, bools, numbers, bytes, strings, ellipsis etc are
     # all constants instead of individual ast nodes.
     return self._match_constant(node, node.s)
 
+  def match_Ellipsis(self, node):
+    return self._match_constant(node, Ellipsis)
+
+  def match_FunctionDef(self, node):
+    symbol = _SymbolMatcher.from_regex(r"(%s|None)" % node.name)
+    return [
+        (self._get_match_location(node, tr.symbol), tr)
+        for tr in self._get_traces(node.lineno, ["MAKE_FUNCTION"], symbol, 1)
+    ]
+
   def match_Import(self, node):
     return list(self._match_import(node, is_from=False))
 
@@ -333,7 +340,7 @@ class MatchAstVisitor(visitor.BaseVisitor):
 class _LineNumberVisitor(visitor.BaseVisitor):
 
   def __init__(self, *args, **kwargs):
-    super(_LineNumberVisitor, self).__init__(*args, **kwargs)
+    super().__init__(*args, **kwargs)
     self.line = 0
 
   def generic_visit(self, node):
