@@ -1,14 +1,11 @@
-# python3
-
 """Generate cross references from a project."""
 
-from __future__ import print_function
-
 import collections
 import re
 from typing import Optional
 
 import attr
+
 from pytype import abstract
 from pytype import analyze
 from pytype import config
@@ -16,12 +13,12 @@ from pytype import errors
 from pytype import io
 from pytype import load_pytd
 from pytype import module_utils
+from pytype.ast import visitor as ast_visitor
 from pytype.pytd import pytd
 from pytype.pytd import pytd_utils
 from pytype.pytd import visitors
 from pytype.tools.traces import source
 from pytype.tools.traces import traces
-from pytype.tools.traces import visitor as ast_visitor
 
 from pytype.tools.xref import callgraph
 from pytype.tools.xref import utils as xref_utils
@@ -101,7 +98,7 @@ class AttrError(Exception):
 
 
 @attr.s
-class PytypeValue(object):
+class PytypeValue:
   """Stores a value inferred by pytype."""
 
   module = attr.ib()
@@ -158,7 +155,7 @@ class PytypeValue(object):
 
 
 @attr.s
-class Module(object):
+class Module:
   """Module representation."""
   name = attr.ib()
 
@@ -171,7 +168,7 @@ class Module(object):
 
 
 @attr.s
-class DocString(object):
+class DocString:
   """Store the text and location of a docstring."""
 
   text = attr.ib()
@@ -198,7 +195,7 @@ class DocString(object):
 
 
 @attr.s
-class Definition(object):
+class Definition:
   """A symbol definition.
 
   Attributes:
@@ -260,7 +257,7 @@ class Definition(object):
 
 
 @attr.s
-class Remote(object):
+class Remote:
   """A symbol from another module."""
 
   module = attr.ib()
@@ -285,7 +282,7 @@ class Remote(object):
 
 
 @attr.s
-class DefLocation(object):
+class DefLocation:
   """A location of a symbol definition.
 
   Attributes:
@@ -301,7 +298,7 @@ class DefLocation(object):
 
 
 @attr.s
-class FunctionParam(object):
+class FunctionParam:
   """A link between a function def and the defs of its params."""
 
   def_id = attr.ib(type=str)
@@ -310,7 +307,7 @@ class FunctionParam(object):
 
 
 @attr.s
-class Reference(object):
+class Reference:
   """A symbol holding a reference to a definition.
 
   Attributes:
@@ -341,7 +338,7 @@ class Reference(object):
 
 
 @attr.s
-class NameArg(object):
+class NameArg:
   """Representation of a single-variable function call argument."""
 
   name = attr.ib()
@@ -349,7 +346,7 @@ class NameArg(object):
 
 
 @attr.s
-class ExprArg(object):
+class ExprArg:
   """Representation of an expression function call argument."""
 
   names = attr.ib()
@@ -357,7 +354,7 @@ class ExprArg(object):
 
 
 @attr.s
-class Funcall(object):
+class Funcall:
   """Representation of a function call."""
 
   name = attr.ib()
@@ -368,7 +365,7 @@ class Funcall(object):
   return_type = attr.ib()
 
 
-class Env(object):
+class Env:
   """A collection of namespaced symbols."""
 
   def __init__(self, ast, scope, parent, cls):
@@ -442,18 +439,6 @@ class Env(object):
 # Also names like visit_Name are self-documenting and do not need docstrings.
 
 
-class CollectNamesVisitor(ast_visitor.BaseVisitor):
-  """Collect all occurences of Name in a subtree."""
-
-  def __init__(self, ast):
-    super(CollectNamesVisitor, self).__init__(ast)
-    self.names = []
-
-  def visit_Name(self, node):
-    if isinstance(node.ctx, self._ast.Load):
-      self.names.append(node)
-
-
 class ScopedVisitor(ast_visitor.BaseVisitor):
   """An AST node visitor that keeps track of scopes and environments.
 
@@ -467,7 +452,7 @@ class ScopedVisitor(ast_visitor.BaseVisitor):
   # anything by way of maintainability or readability?
 
   def __init__(self, ast, module_name, **kwargs):
-    super(ScopedVisitor, self).__init__(ast=ast, **kwargs)  # pytype: disable=wrong-keyword-args
+    super().__init__(ast=ast, **kwargs)  # pytype: disable=wrong-keyword-args
     self.stack = []
     self.class_ids = []
     self.envs = {}
@@ -533,12 +518,12 @@ class ScopedVisitor(ast_visitor.BaseVisitor):
     self.add_scope(node)
 
   def enter_Module(self, node):
-    super(ScopedVisitor, self).enter_Module(node)  # pytype: disable=attribute-error
+    super().enter_Module(node)  # pytype: disable=attribute-error
     self.add_scope(node)
 
   def leave(self, node):
     """If the node has introduced a new scope, we need to pop it off."""
-    super(ScopedVisitor, self).leave(node)
+    super().leave(node)
     if node == self.stack[-1]:
       self.stack.pop()
 
@@ -547,9 +532,7 @@ class IndexVisitor(ScopedVisitor, traces.MatchAstVisitor):
   """Visitor that generates indexes."""
 
   def __init__(self, ast, src, module_name):
-    super(IndexVisitor, self).__init__(ast=ast,
-                                       src_code=src,
-                                       module_name=module_name)
+    super().__init__(ast=ast, src_code=src, module_name=module_name)
     self.defs = {}
     self.locs = collections.defaultdict(list)
     self.refs = []
@@ -589,7 +572,7 @@ class IndexVisitor(ScopedVisitor, traces.MatchAstVisitor):
     if isinstance(node, str):
       # We replace nodes with their names after visiting them.
       return node
-    return super(IndexVisitor, self)._get_node_name(node)
+    return super()._get_node_name(node)
 
   def make_def(self, node, **kwargs):
     """Make a definition from a node."""
@@ -705,19 +688,22 @@ class IndexVisitor(ScopedVisitor, traces.MatchAstVisitor):
           ("STORE_NAME", class_name),
           # Classes defined within a function generate a STORE_FAST op.
           ("STORE_FAST", class_name),
+          # A class being declared global anywhere generates a STORE_GLOBAL op.
+          ("STORE_GLOBAL", class_name),
       ])
       # pytype sometimes analyses this twice, leading to duplicate opcode
       # traces. We only want the first two in the list.
       if (len(ops) >= 2 and
           ops[0][0] == "LOAD_BUILD_CLASS" and
-          ops[1][0] in ("STORE_NAME", "STORE_FAST")):
+          ops[1][0] in ("STORE_NAME", "STORE_FAST", "STORE_GLOBAL")):
         _, _, data = ops[1]
         d = _unwrap(data)
+
     assert d, "Did not get pytype data for class %s" % class_name
     defn = self.add_local_def(node, data=data,
                               doc=DocString.from_node(self._ast, node))
     self.classmap[d[0]] = defn
-    super(IndexVisitor, self).enter_ClassDef(node)
+    super().enter_ClassDef(node)
 
   def enter_FunctionDef(self, node):
     last_line = max(node.lineno, node.body[0].lineno - 1)
@@ -777,9 +763,8 @@ class IndexVisitor(ScopedVisitor, traces.MatchAstVisitor):
 
   def visit_Call(self, node):
     name = self._get_node_name(node)
-    name_visitor = CollectNamesVisitor(self._ast)
-    name_visitor.visit(node)
-    arg_varnames = [x.id for x in name_visitor.names]
+    # We have replaced Name() in args with the corresponding string
+    arg_varnames = [x for x in node.args if isinstance(x, str)]
     seen = set()
     for _, (_, _, data) in self.match(node):
       call, return_type = data
@@ -911,7 +896,7 @@ class IndexVisitor(ScopedVisitor, traces.MatchAstVisitor):
 # pylint: enable=missing-docstring
 
 
-class Indexer(object):
+class Indexer:
   """Runs the indexer visitor and collects its results."""
 
   def __init__(self,
@@ -1180,7 +1165,7 @@ class Indexer(object):
 
   def get_pytd_def(self, data, name):
     assert self.vm, "Indexer vm has not been preserved."
-    node = self.vm.root_cfg_node
+    node = self.vm.root_node
     return self.vm.convert.pytd_convert.value_to_pytd_def(node, data, name)
 
   def get_pytd(self, datum):
@@ -1237,7 +1222,7 @@ class VmTrace(source.AbstractTrace):
     types_repr = tuple(
         t and [node_utils.typename(x) for x in t]
         for t in self.types)
-    return "%s %s" % (super(VmTrace, self).__repr__(), types_repr)
+    return "%s %s" % (super().__repr__(), types_repr)
 
 
 def process_file(options, source_text=None, generate_callgraphs=False,
