@@ -3,11 +3,14 @@
 import collections
 import contextlib
 import itertools
+import os.path
 import re
 import subprocess
 import sys
 import threading
+import traceback
 import types
+from typing import List, Tuple
 import weakref
 
 from pytype import pytype_source_utils
@@ -18,6 +21,21 @@ from pytype import pytype_source_utils
 USE_ANNOTATIONS_BACKPORT = False
 
 
+# We disable the check that keeps pytype from running on not-yet-supported
+# versions when we detect that a pytype test is executing, in order to be able
+# to test upcoming versions.
+def _validate_python_version_upper_bound():
+  for frame_summary in traceback.extract_stack():
+    head, tail = os.path.split(frame_summary.filename)
+    if "/pytype/" in head + "/" and (
+        tail.startswith("test_") or tail.endswith("_test.py")):
+      return False
+  return True
+
+
+_VALIDATE_PYTHON_VERSION_UPPER_BOUND = _validate_python_version_upper_bound()
+
+
 def message(error):
   """A convenience function which extracts a message from an exception.
 
@@ -39,7 +57,7 @@ class UsageError(Exception):
 
 def format_version(python_version):
   """Format a version tuple into a dotted version string."""
-  return ".".join([str(x) for x in python_version])
+  return ".".join(str(x) for x in python_version)
 
 
 def version_from_string(version_string):
@@ -91,7 +109,7 @@ def validate_version(python_version):
     # that typing.py isn't introduced until 3.5, anyway.
     raise UsageError(
         "Python versions 3.0 - 3.4 are not supported. Use 3.5 and higher.")
-  elif python_version > (3, 8):
+  elif python_version > (3, 8) and _VALIDATE_PYTHON_VERSION_UPPER_BOUND:
     # We have an explicit per-minor-version mapping in opcodes.py
     raise UsageError("Python versions > 3.8 are not yet supported.")
 
@@ -154,20 +172,22 @@ def concat_tuples(tuples):
   return tuple(itertools.chain.from_iterable(tuples))
 
 
-def get_python_exe(python_version):
-  """Automatically infer the --python_exe argument.
+def get_python_exe(python_version) -> Tuple[List[str], List[str]]:
+  """Find a python executable to use.
 
   Arguments:
     python_version: the version tuple (e.g. (2, 7))
   Returns:
-    A tuple of the inferred python_exe argument and any command-line flags
+    A tuple of the path to the executable and any command-line flags
   """
   # Use custom interpreters, if provided, in preference to the ones in $PATH
   custom_python_exe = pytype_source_utils.get_custom_python_exe(python_version)
   if custom_python_exe:
-    python_exe = custom_python_exe
+    python_exe = [custom_python_exe]
+  elif sys.platform == "win32":
+    python_exe = ["py", "-%d.%d" % python_version]
   else:
-    python_exe = "python%d.%d" % python_version
+    python_exe = ["python%d.%d" % python_version]
   if USE_ANNOTATIONS_BACKPORT and python_version == (2, 7):
     flags = ["-T"]
   else:
@@ -175,7 +195,7 @@ def get_python_exe(python_version):
   return python_exe, flags
 
 
-def get_python_exe_version(python_exe):
+def get_python_exe_version(python_exe: List[str]):
   """Determine the major and minor version of given Python executable.
 
   Arguments:
@@ -185,7 +205,7 @@ def get_python_exe_version(python_exe):
   """
   try:
     python_exe_version = subprocess.check_output(
-        [python_exe, "-V"], stderr=subprocess.STDOUT).decode()
+        python_exe + ["-V"], stderr=subprocess.STDOUT).decode()
   except subprocess.CalledProcessError:
     return None
 
@@ -233,7 +253,7 @@ def _arg_names(f):
   return f.__code__.co_varnames[:f.__code__.co_argcount]
 
 
-class memoize(object):  # pylint: disable=invalid-name
+class memoize:  # pylint: disable=invalid-name
   """A memoizing decorator that supports expressions as keys.
 
   Use it like this:
@@ -277,8 +297,8 @@ class memoize(object):  # pylint: disable=invalid-name
       defaults = {}
     pos_and_arg_tuples = list(zip(range(f.__code__.co_argcount), argnames))
     shared_dict = {}
-    # TODO(kramm): Use functools.wraps or functools.update_wrapper to preserve
-    # the metadata of the original function.
+    # TODO(b/159037011): Use functools.wraps or functools.update_wrapper to
+    # preserve the metadata of the original function.
     def call(*posargs, **kwargs):
       """Call a memoized function."""
       if kwargs or defaults:
@@ -322,7 +342,18 @@ def invert_dict(d):
   return inverted
 
 
-class DynamicVar(object):
+def unique_list(xs):
+  """Return a unique list from an iterable, preserving order."""
+  seen = set()
+  out = []
+  for x in xs:
+    if x not in seen:
+      seen.add(x)
+      out.append(x)
+  return out
+
+
+class DynamicVar:
   """A dynamically scoped variable.
 
   This is a per-thread dynamic variable, with an initial value of None.
@@ -356,7 +387,7 @@ class DynamicVar(object):
     return self._values()[-1]
 
 
-class AnnotatingDecorator(object):
+class AnnotatingDecorator:
   """A decorator for storing function attributes.
 
   Attributes:
@@ -373,7 +404,7 @@ class AnnotatingDecorator(object):
     return decorate
 
 
-class VirtualMachineWeakrefMixin(object):
+class VirtualMachineWeakrefMixin:
 
   __slots__ = ["vm_weakref"]
 
